@@ -19,7 +19,7 @@ This is accomplished by running the script or code in debug mode, "perl -d:Agent
 use strict;
 use warnings;
 use 5.34.0;
-our $VERSION=.003;
+our $VERSION=.004;
 
 our %VER_FIX;
 BEGIN {
@@ -87,11 +87,8 @@ BEGIN {
     $^P=$^P & ($^P ^ $opt);
   }
 
-  # may or may not be compiled with -Dxx option
-  eval { $^D =0};
 }
 
-
 =head1 Agent interface
 
 The Agent interface is implemented via an on demand debugger that can be turned on/off on the fly.  Also it is possible to run multiple different debugger instances with diffrent configurations on different blocks of code.   This was done intentionally as perl is fairly complex in its nature and an agent interface isn't very useful unless it is flexible.
@@ -121,7 +118,6 @@ Inside you script you can issue the following
   # to dump a very human readable full trace
   print Dumper($db->trace);
 
-
 But altering you code is far from ideal.
 
 Another option is to load the agent and a module that pre-configures it for use such as  the L<Devel::Trace::EveryThing> module, wich provides a stack trace to STDERR in real time as frames begin and exit.
@@ -130,6 +126,59 @@ Example using: Devel::Trace::EveryThing
 
   perl -Ilib -d:Agent -MDevel::Agent::EveryThing examples/everything.pl
 
+=head1 Classes that are Agent Aware
+
+Any class that implements the $instance->___db_stack_filter($agent,$frame,$args,$raw_caller) can filter its own current frame prior to execution, or even prevent the frame from being traced at all.
+
+The ___db_stack_filter method is expected to return true, if the call returns false, then the frame should not be traced. Since the frame passed in before it's runtime execution, the duration value will not be set. 
+
+A basic implementation that exposes only the top level calls is defined in L<Devel::Agent::AwareRole>.  Loading this role into your class will hide all calls made by your class, but not calls made directly to it, this includes child classes that make calls to other classes.
+
+Example:
+
+  package My::Class::That::IS::Mostly::Hidden;
+
+  use Role::Tiny::With; # you can also use Moo Moose or other role implementations
+  with 'Devel::Agent::AwareRole';
+
+  1;
+
+If you want to force a class to not show its internals.. say a class like LWP::UserAgent.
+
+  use LWP::UserAgent;
+  reuqire Devel::Agent::AwareRole;
+
+  # now only the top level calls to LWP::UserAgent will show up
+  *LWP::UserAgent::___db_stack_filter=\&Devel::Agent::AwareRole::___db_stack_filter;
+
+Or if you need to disable filtering on a class that has filtering then you can do the opposite
+  
+  *LWP::UserAgent::___db_stack_filter=sub { 1}
+
+To be fully ignored
+
+  *LWP::UserAgent::___db_stack_filter=sub { 0}
+
+=head1 Frame information
+
+The following hash represents what is provided as a representation of a frame
+
+  {
+    caller_class=>'main',         # class that called this class
+    calls=>[],                    # child frames, empty unless $self->save_to_stack is true
+    class_method=>'main::test_a', # the resolved class::method
+    depth=>1,                     # stack depth, 1 is considered the root
+    duration=>undef|Float,        # how long the frame took to execute, only defined when the frame has executed
+    end_id=>undef|Int,            # frame final execution order where in the stack it ended 
+    line=>2,                      # line number the frame was called from
+    no_frame=>0|1,                # when true, this frame would have been filtered but was included for completeness
+    order_id=>1,                  # inital frame execution order, where in the stack it started
+    owner_id=>0,                  # which order_id frame triggered the execution of this frame
+    raw_method=>'main::test_a',   # un-resolved method name
+    source=>'test.pl',            # the source file
+    t0=>[0,0],                    # Frame Start timestamp in: epoch, microseconds
+  }
+
 =head1 DB Constructor options
 
 This section documents the %args the be passed to the new DB(%args) or DB->new(%args) call.  For each option documented in this section, there is an accesor by that given name that can be called by $self->$name($new_value) or my $current_value=$self->$name.
@@ -140,12 +189,10 @@ This section documents the %args the be passed to the new DB(%args) or DB->new(%
 package 
   DB;
 
-# ADD THIS TO THE SYNOPSIS!!!
-# PERL5OPT='-d:Agent'
-
 #use Modern::Perl;
 use strict;
 use warnings;
+require Scalar::Util;
 
 # as easy as Moo makes things.. its not welcome in a debugger ;(
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -224,7 +271,6 @@ sub DEFAULT_DEPTH () { 4 }
 our @EXCLUDE_DEFAULTS=(
   __PACKAGE__,
   qw(
-    DB
     Data::Dumper
     Time::HiRes
     Method::Generate::Accessor
@@ -486,6 +532,18 @@ has tid=>(
   },
 );
 
+=item * agent_aware=>Bool
+
+This enables or disables the use of $self->_agent_aware(...) method.  See: $self->_agent_aware.  Default is true.
+
+=cut
+
+has agent_aware=>(
+  is=>'ro',
+  default=>1,
+  #isa=>Bool,
+);
+
 =item * existing_trace=>Maybe[InstanceOf['DB']]
 
 This acts as a save point for another existing trace to be exected.
@@ -502,7 +560,7 @@ has existing_trace=>(
 
 =item * process_result=>CodeRef
 
-This callback can be used to evaluate/modify the results of a callback. 
+This callback can be used to evaluate/modify the results of a traced method in this callback.
 
 Example callback
 
@@ -549,6 +607,8 @@ Default always returns true
     return 1; 
   }
 
+This is more or less a global frame filter, see: Classes that are Agent Aware
+
 =cut
 
 has filter_on_args=>(
@@ -661,7 +721,10 @@ sub close_depth {
   my ($self,$depth)=@_;
 
   # work around for the $self->stop_trace;
-  return unless defined $self->depths->[$depth];
+  unless(defined $self->depths->[$depth]) {
+    print "$depth is not defined\n";
+    return;
+  }
 
   my $last=pop $self->depths->@*;
   my $t0=$last->{t0};
@@ -743,12 +806,46 @@ sub filter{
     return;
   }
 
+  if($self->agent_aware) {
+    unless($self->_agent_aware($last,$args,$raw_caller)) {
+      $self->restore_trace;
+      return;
+    }
+  }
+
   $self->push_to_stack($last);
   my $level=$self->level;
   push $level->[$level->$#*]->@*,$self->last_depth;
   $self->restore_trace;
 }
 
+=head2 $self->_agent_aware($frame,$args,$raw_caller)
+
+This method is called before a frame is traced if $self->agent_aware is set to true( the default ).  The objective of this method is see of the first argument being passed to this method is a blessed instance of that class.  When the first argument passed to the class is a blessed object that DOES this class then a call to $args->[0]->___db_stack_filter($agent,$frame,$args,$raw_caller) is made.  This allows classes to modify or inspect the frame that will be used in the reace.  If the call to $args->[0]->___db_stack_filter($agent,$frame,$args,$raw_caller)  returns false, the frame is skipped durring the trace period.
+
+Note note Note:
+
+The call to $args->[0]->___db_stack_filter($agent,$frame,$args,$raw_caller) is never wrapped in a eval and should never do anything heavy or it will impact the metrics provided by the debugger.
+
+=cut
+
+sub _agent_aware {
+  my ($self,$frame,$args,$raw_caller)=@_;
+  
+  return 1 unless $args->$#*>-1 && defined($args->[0]);
+  my $class=&Scalar::Util::blessed($args->[0]);
+  return 1 unless $class;
+  my $cb=$class->can('___db_stack_filter');
+  return 1 unless $cb;
+
+  if($frame->{class_method}=~ /^(.*)::/s) {
+    return 1 unless $args->[0]->DOES($1);
+    return $args->[0]->$cb(@_);
+  }
+
+  return 1;
+}
+
 =head2 Self->save_to($frame)
 
 Saves the frame to the stack trace at the proper depth.
@@ -978,8 +1075,8 @@ sub close_sub {
   }
   my $depth=$level->[1];
   my $last=$self->depths->[$depth];
-  $self->process_result->($self,$res,$last);
   $self->close_to($depth);
+  $self->process_result->($self,$res,$last);
   $self->restore_trace;
 }
 
@@ -1112,12 +1209,11 @@ __END__
 
 =head1 Compile time notes
 
-For perl 5.34.0
+For perl 5.34.0+
 
 When loading this moduel All features of the debugger are disabled aside from: ( 0x01, 0x02, and 0x20 ) which are requried to force the execution of DB::DB. Please see the perldoc perlvar and the $PERLDB section.
 
   Which means:    $^P==35
-  Also as a note: $^D==0
 
 =head1 RUNTIME
 
@@ -1126,13 +1222,24 @@ At runtime, this modue tries to exectue $Devel::Agent::AGENT->filter($caller,$ar
   $caller: is the caller information
   $args:   contains an array reference that represents the arguments passed to a given method
 
-=head1 Silly stuff
+=head1 TODO
 
-Please forgive the typos, this was written on holiday in my spare time.
+  1. Add Plack periodic trace implementation/example
+  2. Add Dancer2 periodic trace implementation/example
 
 =head1 AUTHOR
 
 Michael Shipper L<AKALINUX@CPAN.ORG>
 
+=head1 Silly stuff
+
+Please forgive the typos, this was written on holiday in my spare time.
+
+=head1 See Also
+
+Lots of these internals are based on the following:
+
+L<DB>,L<Devel::Trace>,L<perldebguts>
+
 =cut
 
